#include <libxml/parser.h>
#include <libxml/xpath.h>

xmlXPathObjectPtr getnodeset(xmlDocPtr doc, xmlChar *xpath)
{
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;
    context = xmlXPathNewContext(doc);
    if (!context)
    {
	return NULL;
    }
    result = xmlXPathEvalExpression(xpath, context);
    xmlXPathFreeContext(context);
    if (result == NULL)
    {
	return NULL;
    }
    if (xmlXPathNodeSetIsEmpty(result->nodesetval))
    {
	xmlXPathFreeObject(result);
	return NULL;
    }
    return result;
}

int main(int argc, char **argv)
{
    if (argc <= 2)
    {
	printf("Usage:\t%s <doc> <path>\n\t%s <doc> <path/@attr>\n", argv[0], argv[0]);
	return(-1);
    }
    
    xmlDocPtr doc = xmlParseFile((xmlChar*)argv[1]);
    if (!doc)
    {
	return(-2);
    }
    
    xmlChar *path = xmlCharStrdup((xmlChar*) argv[2]);
    xmlXPathObjectPtr result = getnodeset(doc, path);

    if (!result)
    {
	return(-3);
    }
    
    xmlNodeSetPtr nodeset = result->nodesetval;
    int i;
    int l = nodeset->nodeNr;
    for (i = 0; i < l; ++i)
    {
	xmlNodePtr node = nodeset->nodeTab[i];
	switch (node->type)
	{
	case XML_TEXT_NODE:
	case XML_ELEMENT_NODE:
	case XML_ATTRIBUTE_NODE:
	    {
		 printf("%s\n", xmlXPathCastNodeToString(node));
	    } break;
	}
    }
    xmlXPathFreeObject(result);
    
    xmlFree(path);
    xmlFreeDoc(doc);
    xmlCleanupParser();
    return(0);
}
